# Detect Debian users running the script with "sh" instead of bash
if readlink /proc/$$/exe | grep -q "dash"; then
	echo "This script needs to be run with bash, not sh"
	exit
fi

if [[ "$EUID" -ne 0 ]]; then
	echo "Sorry, you need to run this as root"
	exit
fi

if [[ ! -e /dev/net/tun ]]; then
	echo "The TUN device is not available
You need to enable TUN before running this script"
	exit
fi

if [[ -e /etc/debian_version ]]; then
	OS=debian
	GROUPNAME=nogroup
	RCLOCAL='/etc/rc.local'
elif [[ -e /etc/centos-release || -e /etc/redhat-release ]]; then
	OS=centos
	GROUPNAME=nobody
	RCLOCAL='/etc/rc.d/rc.local'
else
	echo "Looks like you aren't running this installer on Ubuntu or CentOS"
	exit
fi

if [[ -e /etc/qibovpn/vpncl.conf ]]; then
	while :
	do
	clear
		echo "Looks like qibovpn is already installed."
		echo
		echo "What do you want to do?"
		echo "   1) Remove qibovpn"
		echo "   2) Exit"
		read -p "Select an option [1-2]: " option
		case $option in
			1) 
			echo
			read -p "Do you really want to remove qibovpn? [y/N]: " -e REMOVE
			if [[ "$REMOVE" = 'y' || "$REMOVE" = 'Y' ]]; then
				if firewall=$(pgrep firewalld); then
					IP=$(firewall-cmd --direct --get-rules ipv4 nat POSTROUTING | grep '\-s 10.2.0.0/24 '"'"'!'"'"' -d 10.2.0.0/24 -j SNAT --to ' | cut -d " " -f 10)
					# Using both permanent and not permanent rules to avoid a firewalld reload.
					firewall-cmd --zone=public --remove-port=1194/udp
					firewall-cmd --zone=trusted --remove-source=10.2.0.0/24
					firewall-cmd --permanent --zone=public --remove-port=1194/udp
					firewall-cmd --permanent --zone=trusted --remove-source=10.2.0.0/24
					firewall-cmd --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
					firewall-cmd --permanent --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
					
					firewall-cmd --zone=public --remove-port=6099/udp
					firewall-cmd --zone=trusted --remove-source=10.3.0.0/24
					firewall-cmd --permanent --zone=public --remove-port=6099/udp
					firewall-cmd --permanent --zone=trusted --remove-source=10.3.0.0/24
					firewall-cmd --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
					firewall-cmd --permanent --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
				else
					IP1=$(grep 'iptables -t nat -A POSTROUTING -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to ' $RCLOCAL | cut -d " " -f 14)
					iptables -t nat -D POSTROUTING -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP1
					sed -i '/iptables -t nat -A POSTROUTING -s 10.2.0.0\/24 ! -d 10.2.0.0\/24 -j SNAT --to /d' $RCLOCAL
					if iptables -L -n | grep -qE '^ACCEPT'; then
						iptables -D INPUT -p udp --dport 1194 -j ACCEPT
						iptables -D FORWARD -s 10.2.0.0/24 -j ACCEPT
						iptables -D FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
						sed -i "/iptables -I INPUT -p $PROTOCOL --dport $PORT -j ACCEPT/d" $RCLOCAL
						sed -i "/iptables -I FORWARD -s 10.2.0.0\/24 -j ACCEPT/d" $RCLOCAL
						sed -i "/iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT/d" $RCLOCAL
					fi
					
					IP2=$(grep 'iptables -t nat -A POSTROUTING -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to ' $RCLOCAL | cut -d " " -f 14)
					iptables -t nat -D POSTROUTING -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP2
					sed -i '/iptables -t nat -A POSTROUTING -s 10.3.0.0\/24 ! -d 10.3.0.0\/24 -j SNAT --to /d' $RCLOCAL
					if iptables -L -n | grep -qE '^ACCEPT'; then
						iptables -D INPUT -p udp --dport 6099 -j ACCEPT
						iptables -D FORWARD -s 10.3.0.0/24 -j ACCEPT
						iptables -D FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
						sed -i "/iptables -I INPUT -p $PROTOCOL --dport $PORT -j ACCEPT/d" $RCLOCAL
						sed -i "/iptables -I FORWARD -s 10.3.0.0\/24 -j ACCEPT/d" $RCLOCAL
						sed -i "/iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT/d" $RCLOCAL
					fi
				fi
				if [[ "$OS" = 'debian' ]]; then
					apt-get remove --purge -y qibovpn #ToDO
				else
					rpm -e qibovpn
				fi
				rm -rf /etc/qibovpn
				rm -rf /usr/local/qibo
				#rm -f /etc/sysctl.d/30-qbvpn-forward.conf 
				echo
				echo "qibovpn removed!"
			else
				echo
				echo "Removal aborted!"
			fi
			exit
			;;
			2) exit;;
		esac
	done
else
	clear
	echo 'Welcome to this qibovpn installer!'
	echo
	# Autodetect IP address and pre-fill for the user
	IP=$(ip addr | grep 'inet' | grep -v inet6 | grep -vE '127\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' | grep -oE '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' | head -1)
	#read -p "IP address: " -e -i $IP IP
	if [[ "$OS" = 'debian' ]]; then
		apt-get update
		apt-get install iptables openssl ca-certificates -y
		apt-get install qibovpn #ToDo
	else
		# Else, the distro is CentOS
		yum install epel-release -y
		yum install iptables openssl openssl-devel lzo lzo-devel pam pam-devel ca-certificates libgcrypt-devel -y
		rpm -ivh RPMS/qibovpn*.rpm
	fi
	# Enable net.ipv4.ip_forward for the system
	SYSCTL='/etc/sysctl.conf'
	sed -i '/\<net.ipv4.ip_forward\>/c\net.ipv4.ip_forward=1' $SYSCTL
	if ! grep -q "\<net.ipv4.ip_forward\>" $SYSCTL; then
		echo 'net.ipv4.ip_forward=1' >> $SYSCTL
	fi
	# Enable without waiting for a reboot or service restart
	echo 1 > /proc/sys/net/ipv4/ip_forward
	if firewall=$(pgrep firewalld); then
		# Using both permanent and not permanent rules to avoid a firewalld
		# reload.
		# We don't use --add-service=QiboVPN because that would only work with
		# the default port and protocol.
		firewall-cmd --zone=public --add-port=1194/udp
		firewall-cmd --zone=trusted --add-source=10.2.0.0/24
		firewall-cmd --permanent --zone=public --add-port=1194/udp
		firewall-cmd --permanent --zone=trusted --add-source=10.2.0.0/24
		# Set NAT for the VPN subnet
		firewall-cmd --direct --add-rule ipv4 nat POSTROUTING 0 -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
		firewall-cmd --permanent --direct --add-rule ipv4 nat POSTROUTING 0 -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
		
		firewall-cmd --zone=public --add-port=6099/udp
		firewall-cmd --zone=trusted --add-source=10.3.0.0/24
		firewall-cmd --permanent --zone=public --add-port=6099/udp
		firewall-cmd --permanent --zone=trusted --add-source=10.3.0.0/24
		# Set NAT for the VPN subnet
		firewall-cmd --direct --add-rule ipv4 nat POSTROUTING 0 -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
		firewall-cmd --permanent --direct --add-rule ipv4 nat POSTROUTING 0 -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
		
	else
		# Needed to use rc.local with some systemd distros
		if [[ "$OS" = 'debian' && ! -e $RCLOCAL ]]; then
			echo '#!/bin/sh -e
exit 0' > $RCLOCAL
		fi
		chmod +x $RCLOCAL
		# Set NAT for the VPN subnet
		iptables -t nat -A POSTROUTING -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
		sed -i "1 a\iptables -t nat -A POSTROUTING -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP" $RCLOCAL
		if iptables -L -n | grep -qE '^(REJECT|DROP)'; then
			# If iptables has at least one REJECT rule, we asume this is needed.
			# Not the best approach but I can't think of other and this shouldn't
			# cause problems.
			iptables -I INPUT -p udp --dport 1194 -j ACCEPT
			iptables -I FORWARD -s 10.2.0.0/24 -j ACCEPT
			iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
			sed -i "1 a\iptables -I INPUT -p $PROTOCOL --dport $PORT -j ACCEPT" $RCLOCAL
			sed -i "1 a\iptables -I FORWARD -s 10.2.0.0/24 -j ACCEPT" $RCLOCAL
			sed -i "1 a\iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT" $RCLOCAL
		fi
		
		iptables -t nat -A POSTROUTING -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
		sed -i "1 a\iptables -t nat -A POSTROUTING -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP" $RCLOCAL
		if iptables -L -n | grep -qE '^(REJECT|DROP)'; then
			# If iptables has at least one REJECT rule, we asume this is needed.
			# Not the best approach but I can't think of other and this shouldn't
			# cause problems.
			iptables -I INPUT -p udp --dport 6099 -j ACCEPT
			iptables -I FORWARD -s 10.3.0.0/24 -j ACCEPT
			iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
			sed -i "1 a\iptables -I INPUT -p $PROTOCOL --dport $PORT -j ACCEPT" $RCLOCAL
			sed -i "1 a\iptables -I FORWARD -s 10.3.0.0/24 -j ACCEPT" $RCLOCAL
			sed -i "1 a\iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT" $RCLOCAL
		fi
	fi
	# And finally, restart qibovpn
	if [[ "$OS" = 'debian' ]]; then
		/etc/init.d/qibovpn restart
	else
		service qibovpn restart
		chkconfig qibovpn on
	fi
	echo
	echo "Finished!"
	echo

fi
