# Detect Debian users running the script with "sh" instead of bash
if readlink /proc/$$/exe | grep -q "dash"; then
	echo "This script needs to be run with bash, not sh"
	exit
fi

if [[ "$EUID" -ne 0 ]]; then
	echo "Sorry, you need to run this as root"
	exit
fi

if [[ ! -e /dev/net/tun ]]; then
	echo "The TUN device is not available
You need to enable TUN before running this script"
	exit
fi

if [[ -e /etc/debian_version ]]; then
	OS=debian
	GROUPNAME=nogroup
	RCLOCAL='/etc/rc.local'
elif [[ -e /etc/centos-release || -e /etc/redhat-release ]]; then
	OS=centos
	GROUPNAME=nobody
	RCLOCAL='/etc/rc.d/rc.local'
else
	echo "Looks like you aren't running this installer on Ubuntu or CentOS"
	exit
fi

if [[ -e /etc/qibovpn/vpncl.conf ]]; then
	while :
	do
	clear
		echo "Looks like qibovpn is already installed."
		echo
		read -p "Do you really want to remove qibovpn? [y/N]: " -e REMOVE
		if [[ "$REMOVE" = 'y' || "$REMOVE" = 'Y' ]]; then
			if firewall=$(pgrep firewalld); then
				IP=$(firewall-cmd --direct --get-rules ipv4 nat POSTROUTING | grep '\-s 10.2.0.0/24 '"'"'!'"'"' -d 10.2.0.0/24 -j SNAT --to ' | cut -d " " -f 10)
				# Using both permanent and not permanent rules to avoid a firewalld reload.
				firewall-cmd --zone=public --remove-port=1194/udp
				firewall-cmd --zone=trusted --remove-source=10.2.0.0/24
				firewall-cmd --permanent --zone=public --remove-port=1194/udp
				firewall-cmd --permanent --zone=trusted --remove-source=10.2.0.0/24
				firewall-cmd --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
				firewall-cmd --permanent --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP
				
				firewall-cmd --zone=public --remove-port=6099/udp
				firewall-cmd --zone=trusted --remove-source=10.3.0.0/24
				firewall-cmd --permanent --zone=public --remove-port=6099/udp
				firewall-cmd --permanent --zone=trusted --remove-source=10.3.0.0/24
				firewall-cmd --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
				firewall-cmd --permanent --direct --remove-rule ipv4 nat POSTROUTING 0 -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP
			else
				IP1=$(grep 'iptables -t nat -A POSTROUTING -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to ' $RCLOCAL | cut -d " " -f 14)
				iptables -t nat -D POSTROUTING -s 10.2.0.0/24 ! -d 10.2.0.0/24 -j SNAT --to $IP1
				sed -i '/iptables -t nat -A POSTROUTING -s 10.2.0.0\/24 ! -d 10.2.0.0\/24 -j SNAT --to /d' $RCLOCAL
				if iptables -L -n | grep -qE '^ACCEPT'; then
					iptables -D INPUT -p udp --dport 1194 -j ACCEPT
					iptables -D FORWARD -s 10.2.0.0/24 -j ACCEPT
					iptables -D FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
					sed -i "/iptables -I INPUT -p udp --dport 1194 -j ACCEPT/d" $RCLOCAL
					sed -i "/iptables -I FORWARD -s 10.2.0.0\/24 -j ACCEPT/d" $RCLOCAL
					sed -i "/iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT/d" $RCLOCAL
				fi
				
				IP2=$(grep 'iptables -t nat -A POSTROUTING -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to ' $RCLOCAL | cut -d " " -f 14)
				iptables -t nat -D POSTROUTING -s 10.3.0.0/24 ! -d 10.3.0.0/24 -j SNAT --to $IP2
				sed -i '/iptables -t nat -A POSTROUTING -s 10.3.0.0\/24 ! -d 10.3.0.0\/24 -j SNAT --to /d' $RCLOCAL
				if iptables -L -n | grep -qE '^ACCEPT'; then
					iptables -D INPUT -p udp --dport 6099 -j ACCEPT
					iptables -D FORWARD -s 10.3.0.0/24 -j ACCEPT
					iptables -D FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
					sed -i "/iptables -I INPUT -p udp --dport 6099 -j ACCEPT/d" $RCLOCAL
					sed -i "/iptables -I FORWARD -s 10.3.0.0\/24 -j ACCEPT/d" $RCLOCAL
					sed -i "/iptables -I FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT/d" $RCLOCAL
				fi
			fi
			if [[ "$OS" = 'debian' ]]; then
				dpkg -P qibovpn
			else
				rpm -e qibovpn
			fi
			rm -rf /etc/qibovpn
			rm -rf /usr/local/qibo
			#rm -f /etc/sysctl.d/30-qbvpn-forward.conf 
			echo
			echo "qibovpn removed!"
		else
			echo
			echo "Removal aborted!"
		fi
		exit
	done
else
	clear
	echo "Looks like qibovpn is not installed."
	echo
	echo "Finished!"
	echo
fi
